@@ -46,11 +46,12 @@ macro_rules! declare {
             where
                 E: Encoder
             {
-                let ($ident0, $($ident),*) = self;
-                let mut pack = encoder.encode_tuple(count!($ident0 $($ident)*))?;
-                <$ty0>::encode($ident0, pack.next()?)?;
-                $(<$ty>::encode($ident, pack.next()?)?;)*
-                pack.end()
+                encoder.encode_tuple(count!($ident0 $($ident)*), |mut pack| {
+                    let ($ident0, $($ident),*) = self;
+                    <$ty0>::encode($ident0, pack.next()?)?;
+                    $(<$ty>::encode($ident, pack.next()?)?;)*
+                    pack.end()
+                })
             }
         }
 
@@ -60,10 +61,11 @@ macro_rules! declare {
             where
                 D: Decoder<'de>
             {
-                let mut unpack = decoder.decode_tuple(count!($ident0 $($ident)*))?;
-                let $ident0 = unpack.next().and_then(<$ty0>::decode)?;
-                $(let $ident = unpack.next().and_then(<$ty>::decode)?;)*
-                Ok(($ident0, $($ident),*))
+                decoder.decode_tuple(count!($ident0 $($ident)*), |mut unpack| {
+                    let $ident0 = unpack.next().and_then(<$ty0>::decode)?;
+                    $(let $ident = unpack.next().and_then(<$ty>::decode)?;)*
+                    Ok(($ident0, $($ident),*))
+                })
             }
         }
 
@@ -73,11 +75,12 @@ macro_rules! declare {
             where
                 E: Encoder
             {
-                let Packed(($ident0, $($ident),*)) = self;
-                let mut pack = encoder.encode_pack()?;
-                <$ty0>::encode($ident0, pack.next()?)?;
-                $(<$ty>::encode($ident, pack.next()?)?;)*
-                pack.end()
+                encoder.encode_pack(|mut pack| {
+                    let Packed(($ident0, $($ident),*)) = self;
+                    <$ty0>::encode($ident0, pack.next()?)?;
+                    $(<$ty>::encode($ident, pack.next()?)?;)*
+                    pack.end()
+                })
             }
         }
 
@@ -87,10 +90,11 @@ macro_rules! declare {
             where
                 D: Decoder<'de>
             {
-                let mut unpack = decoder.decode_pack()?;
-                let $ident0 = unpack.next().and_then(<$ty0>::decode)?;
-                $(let $ident = unpack.next().and_then(<$ty>::decode)?;)*
-                Ok(Packed(($ident0, $($ident),*)))
+                decoder.decode_pack(|mut unpack| {
+                    let $ident0 = unpack.next().and_then(<$ty0>::decode)?;
+                    $(let $ident = unpack.next().and_then(<$ty>::decode)?;)*
+                    Ok(Packed(($ident0, $($ident),*)))
+                })
             }
         }
 
